@@ -1,5 +1,5 @@
 /* Portability cruft.  Include after config.h and sys/types.h.
-   Copyright 1996, 1998-2000, 2007, 2009-2010 Free Software Foundation, Inc.
+   Copyright 1996, 1998-2000, 2007, 2009-2011 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,6 +16,9 @@
    Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
    02110-1301, USA.  */
 
+#ifndef GREP_SYSTEM_H
+#define GREP_SYSTEM_H 1
+
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -54,3 +57,4 @@ enum { EXIT_TROUBLE = 2 };
 #endif
 
 #include "unlocked-io.h"
+#endif
